#
#        windows用HPC/PF実行環境構築スクリプト    ver. 1.2
#
#インストールするソフトのバージョン
$python_version_string="2.7.10"
$wxpython_version_string="3.0.2.0"
$node_version_string="0.12.7"
$pdi_version_string="1.3.2" #PDIの仕様変更のためこれ以降のバージョンは利用不可

#PDIとhpcpfGUIのインストール先
$dest = "C:\WORK\HPCPF"
#アーカイブファイルの一時保存先
$tmp_dir=[Environment]::GetEnvironmentVariable('TMP', 'User')

#
# ダウンロード元URLの設定
#
# hpcpfguiとPDIは32bit/64bitの区別は無いのでurl64のみに指定している
#
$url64 = @{python   = "https://www.python.org/ftp/python/"+$python_version_string+"/python-"+$python_version_string+".amd64.msi";
           wxpython = "http://downloads.sourceforge.net/wxpython/wxPython3.0-win64-"+$wxpython_version_string+"-py27.exe";
           node     = "http://nodejs.org/dist/v"+$node_version_string+"/x64/node-v"+$node_version_string+"-x64.msi";
           hpcpfgui = "https://github.com/avr-aics-riken/hpcpfGUI/archive/master.zip";
           pdi      = "https://github.com/avr-aics-riken/PDI/archive/RELEASE_v"+$pdi_version_string+".zip"}

$url32 = @{python   = "https://www.python.org/ftp/python/"+$python_version_string+"/python-"+$python_version_string+".msi";
           wxpython = "http://downloads.sourceforge.net/wxpython/wxPython3.0-win32-"+$wxpython_version_string+"-py27.exe";
           node     = "http://nodejs.org/dist/v"+$node_version_string+"/node-v"+$node_version_string+"-x86.msi"}

$option = @{python="/quiet";wxpython="/verysilent";node="/passive"}

#
# インストールスクリプト開始
#

$osBitness = (Get-WmiObject -Class Win32_OperatingSystem).OSarchitecture.SubString(0,2)

 ""
 "hpcpfGUI/PDI setup script start"
 " installing $osBitness bit version"
 ""

#PDI/hpcpfGUIのインストールpathに同名の**ファイル**が存在したら
#メッセージを出力して終了する
if (Test-Path $dest -PathType Leaf)
{
  Write-Warning "$dest **file** exists"
    Write-Warning "remove that file or change destination path and rerun this script"
    exit
}


$web = (New-Object System.Net.WebClient)
#
# python/wxpython/node.jsのインストール
#
Foreach ( $i in "python", "wxpython", "node")
{
#アーカイブのダウンロード
  if ($osBitness -eq 64)
  {
    $url = $url64.$i
  }else{
    $url = $url32.$i
  }
  $filename=(Join-Path $tmp_dir (Split-Path ([System.Uri] $url).AbsolutePath -Leaf))
  if(Test-Path $filename -PathType Leaf)
  {
    "$i archive found. try to re-use it" 
  }else{
    "downlod $i from web"
      $web.DownloadFile($url, $filename)
      "Done!"
  }

# インストーラの実行
  "install $i"
  $rt=Start-Process $filename -Wait -PassThru -ArgumentList $option.$i
  if($rt.ExitCode -ne 0)
  {
    Write-Warning "$i install failed !!"
      exit
  }else{
    "Done !"
  }
}

#C:\Python27をPATHに追加
$path = [Environment]::GetEnvironmentVariable('PATH', 'User')
$path_list = $path -split ";"
$has_python = $FALSE
Foreach($i in $path_list)
{
  if($i -eq "C:\Python27")
  {
    $has_python=$TRUE
    echo "Python in PATH"
  }
}
if(! $has_python)
{
  $path += ";C:\Python27"
    [Environment]::SetEnvironmentVariable('PATH', $path, 'User')
}

# PDIとhpcpfGUIのインストール先フォルダを作成
if ( !(Test-Path $dest -PathType Container))
{
  New-Item $dest -type directory
} 

# PDIとhpcpfGUIをzipでダウンロードして展開する
$dest_childs=@{pdi="NULL"; hpcpfgui="NULL"}
Foreach($i in "pdi", "hpcpfgui")
{
#アーカイブの入手
#以前のバージョンではアーカイブの名前が被っていたためリネームしている
  $archive=$i+".zip"
    $filename=(Join-Path $tmp_dir $archive)
    if(Test-Path $filename -PathType Leaf)
    {
      "$i archive found. try to re-use it" 
    }else{
      "download $i from web"
        $web.DownloadFile($url64.$i, $filename)
        "Done!"
    }

#解凍先ディレクトリのバックアップ&新規作成
  $dest_child=(Join-Path $dest  $i)
    $dest_childs.$i=$dest_child
    if (Test-Path $dest_child -PathType Container)
    {
      $old=$dest_child+".old"
      if (Test-Path $old -PathType Container)
      {
        Remove-Item $old -Recurse
      }
      Move-Item $dest_child $old
    }
  New-Item $dest_child -type directory

#zipファイルの解凍
    if ($PSVersionTable.CLRVersion.Major -ge 4)
    {
      Add-Type -assembly "system.io.compression.filesystem"
        [io.compression.zipfile]::ExtractToDirectory($filename, $dest_child)
    }else{
# windows7以前用のfallback
      $shell = new-object -com shell.application
        $zip = $shell.NameSpace($filename)
        foreach($item in $zip.items())
        {
          $shell.Namespace($dest_child).copyhere($item)
        }
    }

}

# hpcpfGUIのインストール
$installer=(Join-Path $dest_child "\hpcpfGUI-master\bin\install.bat")
  echo $installer
if (Test-Path $installer -PathType Leaf)
{
  "install hpcpfGUI"
    $rt=Start-Process install.bat -Wait -PassThru -WorkingDirectory (Split-Path  $installer -Parent)
    if($rt.ExitCode -ne 0)
    {
      Write-Warning "hpcpfGUI install failed !!"
        exit
    }else{
      "Done !"
    }
}else{
  Write-Warning "hpcpfGUI installer is not found !!"
}

#PDIのインストールパスをhpcpfGUI.confに書き込む
$conf_file=(Join-Path $dest_childs['hpcpfgui'] "\hpcpfGUI-master\conf\hpcpfGUI.conf")
$tmp="\PDI-RELEASE_v"+$pdi_version_string+"\bin\pdi_exec.bat"
$pdi_path=(Join-Path $dest_childs['pdi'] $tmp)

#conf_fileとpdi_pathの\を\\で置き換える
$conf_file=$conf_file.Replace("\", "\\")
$pdi_path=$pdi_path.Replace("\", "\\")

if (Test-Path "tmp.py" -PathType Leaf)
{
  Remove-Item "tmp.py"
}
$script="import json
import io

with io.open('$conf_file') as f:
    conf=json.load(f)
    conf['PDI']['win32']='$pdi_path'
    with open('tmp', 'w') as g:
        g.write(json.dumps(conf, indent=4))
"
New-Item -Path "tmp.py" -Type File -Value $script
$rt=Start-Process C:\Python27\python.exe -Wait -PassThru -ArgumentList "tmp.py"
if($rt.ExitCode -ne 0)
{
  "hpcpfGUI.conf rewrite failed!"
}else{
  Remove-Item "tmp.py"
  Move-Item -force "tmp" $conf_file
}

